Stan Math Library: stan/math/prim/scal/prob/rayleigh_log.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

rayleigh_log.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_RAYLEIGH_LOG_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_RAYLEIGH_LOG_HPP
3
4 #include <boost/random/uniform_real_distribution.hpp>
5 #include <boost/random/variate_generator.hpp>
6 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
7 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
8 #include <stan/math/prim/scal/err/check_nonnegative.hpp>
9 #include <stan/math/prim/scal/err/check_not_nan.hpp>
10 #include <stan/math/prim/scal/err/check_positive.hpp>
11 #include <stan/math/prim/scal/fun/log1m.hpp>
12 #include <stan/math/prim/scal/fun/square.hpp>
13 #include <stan/math/prim/scal/fun/value_of.hpp>
14 #include <stan/math/prim/scal/meta/length.hpp>
15 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
16 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
17 #include <stan/math/prim/scal/fun/constants.hpp>
18 #include <stan/math/prim/scal/meta/include_summand.hpp>
19 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
20 #include <cmath>
21
22 namespace stan {
23
24 namespace math {
25
26 template <bool propto,
27 typename T_y, typename T_scale>
28 typename return_type<T_y, T_scale>::type
29 rayleigh_log(const T_y& y, const T_scale& sigma) {
30 static const char* function("stan::math::rayleigh_log");
31 typedef typename stan::partials_return_type<T_y, T_scale>::type
32 T_partials_return;
33
34 using std::log;
35 using stan::is_constant_struct;
36 using stan::math::check_positive;
37 using stan::math::check_not_nan;
38 using stan::math::check_consistent_sizes;
39 using stan::math::value_of;
40 using stan::math::include_summand;
41 using std::log;
42
43 // check if any vectors are zero length
44 if (!(stan::length(y) && stan::length(sigma)))
45 return 0.0;
46
47 // set up return value accumulator
48 T_partials_return logp(0.0);
49
50 // validate args (here done over var, which should be OK)
51 check_not_nan(function, "Random variable", y);
52 check_positive(function, "Scale parameter", sigma);
53 check_positive(function, "Random variable", y);
54 check_consistent_sizes(function,
55 "Random variable", y,
56 "Scale parameter", sigma);
57
58 // check if no variables are involved and prop-to
59 if (!include_summand<propto, T_y, T_scale>::value)
60 return 0.0;
61
62 // set up template expressions wrapping scalars into vector views
63 OperandsAndPartials<T_y, T_scale> operands_and_partials(y, sigma);
64
65 VectorView<const T_y> y_vec(y);
66 VectorView<const T_scale> sigma_vec(sigma);
67 size_t N = max_size(y, sigma);
68
69 VectorBuilder<true, T_partials_return, T_scale> inv_sigma(length(sigma));
70 VectorBuilder<include_summand<propto, T_scale>::value,
71 T_partials_return, T_scale> log_sigma(length(sigma));
72 for (size_t i = 0; i < length(sigma); i++) {
73 inv_sigma[i] = 1.0 / value_of(sigma_vec[i]);
74 if (include_summand<propto, T_scale>::value)
75 log_sigma[i] = log(value_of(sigma_vec[i]));
76 }
77
78 for (size_t n = 0; n < N; n++) {
79 // pull out values of arguments
80 const T_partials_return y_dbl = value_of(y_vec[n]);
81
82 // reusable subexpression values
83 const T_partials_return y_over_sigma = y_dbl * inv_sigma[n];
84
85 static double NEGATIVE_HALF = -0.5;
86
87 // log probability
88 if (include_summand<propto, T_scale>::value)
89 logp -= 2.0 * log_sigma[n];
90 if (include_summand<propto, T_y>::value)
91 logp += log(y_dbl);
92 // if (include_summand<propto, T_y, T_scale>::value)
93 logp += NEGATIVE_HALF * y_over_sigma * y_over_sigma;
94
95 // gradients
96 T_partials_return scaled_diff = inv_sigma[n] * y_over_sigma;
97 if (!is_constant_struct<T_y>::value)
98 operands_and_partials.d_x1[n] += 1.0 / y_dbl - scaled_diff;
99 if (!is_constant_struct<T_scale>::value)
100 operands_and_partials.d_x2[n]
101 += y_over_sigma * scaled_diff - 2.0 * inv_sigma[n];
102 }
103 return operands_and_partials.value(logp);
104 }
105
106 template <typename T_y, typename T_scale>
107 inline
108 typename return_type<T_y, T_scale>::type
109 rayleigh_log(const T_y& y, const T_scale& sigma) {
110 return rayleigh_log<false>(y, sigma);
111 }
112 }
113 }
114 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::log
fvar< T > log(const fvar< T > &x)
Definition: log.hpp:15
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
square.hpp
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::include_summand
Template metaprogram to calculate whether a summand needs to be included in a proportional (log) prob...
Definition: include_summand.hpp:36
stan::return_type::type
boost::math::tools::promote_args< typename scalar_type< T1 >::type, typename scalar_type< T2 >::type, typename scalar_type< T3 >::type, typename scalar_type< T4 >::type, typename scalar_type< T5 >::type, typename scalar_type< T6 >::type >::type type
Definition: return_type.hpp:27
include_summand.hpp
check_nonnegative.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::rayleigh_log
return_type< T_y, T_scale >::type rayleigh_log(const T_y &y, const T_scale &sigma)
Definition: rayleigh_log.hpp:29
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::math::check_positive
bool check_positive(const char *function, const char *name, const T_y &y)
Return true if y is positive.
Definition: check_positive.hpp:68
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
log1m.hpp
partials_return_type.hpp
OperandsAndPartials.hpp
length.hpp
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
constants.hpp
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
stan::partials_return_type::type
boost::math::tools::promote_args< typename partials_type< typename scalar_type< T1 >::type >::type, typename partials_type< typename scalar_type< T2 >::type >::type, typename partials_type< typename scalar_type< T3 >::type >::type, typename partials_type< typename scalar_type< T4 >::type >::type, typename partials_type< typename scalar_type< T5 >::type >::type, typename partials_type< typename scalar_type< T6 >::type >::type >::type type
Definition: partials_return_type.hpp:26
check_consistent_sizes.hpp
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39

[ Stan Home Page ] © 2011–2016, Stan Development Team.